@@ -1,25 +1,29 @@
 from logging import getLogger
+from typing import Optional
 
 from PIL import Image
 
-from ..device_pool import JobContext
 from ..params import ImageParams, Size, StageParams
-from ..utils import ServerContext
+from ..server import ServerContext
+from ..worker import WorkerContext
 
 logger = getLogger(__name__)
 
 
 def reduce_crop(
-    _job: JobContext,
+    _job: WorkerContext,
     _server: ServerContext,
     _stage: StageParams,
     _params: ImageParams,
-    source_image: Image.Image,
+    source: Image.Image,
     *,
     origin: Size,
     size: Size,
+    stage_source: Optional[Image.Image] = None,
     **kwargs,
 ) -> Image.Image:
-    image = source_image.crop((origin.width, origin.height, size.width, size.height))
+    source = stage_source or source
+
+    image = source.crop((origin.width, origin.height, size.width, size.height))
     logger.info("created thumbnail with dimensions: %sx%s", image.width, image.height)
     return image
