@@ -3,30 +3,32 @@ from typing import Callable
 
 from PIL import Image
 
-from ..device_pool import JobContext
 from ..params import ImageParams, Size, StageParams
-from ..utils import ServerContext
+from ..server import ServerContext
+from ..worker import WorkerContext
 
 logger = getLogger(__name__)
 
 
 def source_noise(
-    _job: JobContext,
+    _job: WorkerContext,
     _server: ServerContext,
     _stage: StageParams,
-    params: ImageParams,
-    source_image: Image.Image,
+    _params: ImageParams,
+    source: Image.Image,
     *,
     size: Size,
     noise_source: Callable,
+    stage_source: Image.Image,
     **kwargs,
 ) -> Image.Image:
+    source = stage_source or source
     logger.info("generating image from noise source")
 
-    if source_image is not None:
+    if source is not None:
         logger.warn("a source image was passed to a noise stage, but will be discarded")
 
-    output = noise_source(source_image, (size.width, size.height), (0, 0))
+    output = noise_source(source, (size.width, size.height), (0, 0))
 
     logger.info("final output image size: %sx%s", output.width, output.height)
     return output
